//**************************************************************
// File         : RTDpwl.c
// Author       : Automatically generated using 'coefRTD.exe'
// Compiler     : intended for Keil C51
// Description  : Subroutines for linearization of RTD signals
//                using piecewise linear approximation method.
// More Info    : Details in application note AN-709, available
//                at....  http://www.analog.com/MicroConverter
//**************************************************************

// definitons....
#define TMIN (-48)  // = minimum temperature in degC
#define TMAX (168)  // = maximum temperature in degC
#define RMIN (81.1003)  // = input resistance in ohms at -48 degC
#define RMAX (164.03)  // = input resistance in ohms at 168 degC
#define NSEG 16  // = number of sections in table
#define RSEG 5.18308  // = (RMAX-RMIN)/NSEG = resistance RSEG in ohms of each segment

// lookup table....
const float C_rtd[] = {-48.0036,-34.9137,-21.7667,-8.56524,4.68889,17.9952,31.3543,44.7668,58.2335,71.7549,85.3317,98.9647,112.654,126.402,140.207,154.072,167.996};
// lookup table size:
//   = 16 linear sections
//   = 17 coefficients
//   = 68 bytes (4 bytes per floating point coefficient)

// linearization routine error band:  
//   = -0.00375366degC .. 0.00376551degC
// specified over measurement range -48degC .. 168degC

// _____________________________________________________________
// Temperature of RTD Function                             T_rtd
// input: r = resistance of RTD
// output: T_rtd() = corresponding temperature of RTD
// Calculates temperature of RTD as a function of resistance via
// a piecewise linear approximation method.

float T_rtd (float r) {
  float t;
  int i;
  i=(r-RMIN)/RSEG;       // determine which coefficients to use
  if (i<0)               // if input is under-range..
    i=0;                 // ..then use lowest coefficients
  else if (i>NSEG-1)     // if input is over-range..
    i=NSEG-1;            // ..then use highest coefficients
  t = C_rtd[i]+(r-(RMIN+RSEG*i))*(C_rtd[i+1]-C_rtd[i])/RSEG;
  return (t);
}

// _____________________________________________________________
// Resistance of RTD Function                              R_rtd
// input: t = temperature of RTD
// output: R_rtd() = corresponding resistance of RTD
// Calculates resistance of RTD as a function of temperature via
// a piecewise linear approximation method.

float R_rtd (float t) {
  float r;
  int i, adder;

  // set up initial values
  i = NSEG/2;           // starting value for 'i' index
  adder = (i+1)/2;      // adder value used in do loop

  // determine if input t is within range
  if (t<C_rtd[0])           // if input is under-range..
    i=0;                    // ..then use lowest coefficients
  else if (t>C_rtd[NSEG])   // if input is over-range..
    i=NSEG-1;               // ..then use highest coefficients

  // if input within range, determine which coefficients to use
  else do {
    if (C_rtd[i]>t)   i-=adder; // either decrease i by adder..
    if (C_rtd[i+1]<t) i+=adder; // ..or increase i by adder
    if (i<0)       i=0;         // make sure i is >=0..
    if (i>NSEG-1)  i=NSEG-1;    // ..and <=NSEG-1
    adder = (adder+1)/2;        // divide adder by two (rounded)
  } while ((C_rtd[i]>t)||(C_rtd[i+1]<t));   // repeat 'til done

  // compute final result
  r = RMIN+RSEG*i + (t-C_rtd[i])*RSEG/(C_rtd[i+1]-C_rtd[i]);

  return (r);
}

// _____________________________________________________________
// Minimum Temperature Function                         Tmin_rtd
// Returns minimum temperature specified by lookup table.
float Tmin_rtd () {
  return (TMIN);
}

// _____________________________________________________________
// Maximum Temperature Function                         Tmax_rtd
// Returns maximum temperature specified by lookup table.
float Tmax_rtd () {
  return (TMAX);
}

// _____________________________________________________________
// Minimum Resistance Function                          Rmin_rtd
// Returns minimum RTD resistance specified by lookup table.
float Rmin_rtd () {
  return (RMIN);
}

// _____________________________________________________________
// Maximum Resistance Function                          Rmax_rtd
// Returns maximum RTD resistance specified by lookup table.
float Rmax_rtd () {
  return (RMAX);
}
